'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.setInitialProps = setInitialProps;
exports.diffProps = diffProps;
exports.updateProps = updateProps;

var _objectFactory = require('./object-factory');

var _Registry = require('./Registry');

var isEventRegex = /^on([A-Z][a-zA-Z]+)$/;
var entries = require('lodash/entries');
var keys = require('lodash/keys');

function listenTo(qmlElement, eventName, value, lastValue) {
  eventName = eventName[0].toLowerCase() + eventName.substring(1);


  if (!qmlElement[eventName]) {
    return;
  }

  if (lastValue) {
    qmlElement[eventName].disconnect(lastValue);
  }

  qmlElement[eventName].connect(value);
}

function setInitialProps(qmlElement, nextProps) {

  entries(nextProps).forEach(function (_ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        propKey = _ref2[0],
        propValue = _ref2[1];

    if (propKey === 'children' || propValue == null) {
      // ignore
      return;
    }

    if (propKey === '__qmlRawContent') {
      return;
    }

    if (propKey.match(isEventRegex)) {
      var match = propKey.match(isEventRegex);
      listenTo(qmlElement, match[1], propValue, null);
      return;
    }

    if ((typeof propValue === 'undefined' ? 'undefined' : _typeof(propValue)) === 'object') {
      if (!qmlElement[propKey]) {
        return;
      }

      entries(propValue).forEach(function (_ref3) {
        var _ref4 = _slicedToArray(_ref3, 2),
            configKey = _ref4[0],
            configValue = _ref4[1];

        qmlElement[propKey][configKey] = configValue;
      });
      return;
    }

    qmlElement[propKey] = propValue;
  });
}

function diffProps(qmlElement, lastProps, nextProps) {
  var updatePayload = null;

  var add = function add(k, v) {
    if (!updatePayload) updatePayload = [];
    updatePayload.push([k, v]);
  };

  for (var propKey in keys(lastProps)) {
    if (lastProps[propKey] == null || nextProps.hasOwnProperty(propKey)) {
      continue;
    } else if (propKey.match(isEventRegex)) {
      updatePayload = updatePayload || [];
    }
  }

  entries(nextProps).forEach(function (_ref5) {
    var _ref6 = _slicedToArray(_ref5, 2),
        propKey = _ref6[0],
        nextProp = _ref6[1];

    var lastProp = lastProps[propKey];

    if (nextProp === lastProp || propKey === 'style' || nextProp == null && lastProp == null) {
      return;
    } else if (propKey === 'children' && lastProp !== nextProp) {
      add(propKey, nextProp);
    } else if (propKey.match(isEventRegex) && lastProp !== nextProp) {
      // we need the last event handler so we can remove it in the commit phase
      add(propKey, [lastProp, nextProp]);
    } else {
      // For any other property we always add it to the queue and then we
      // filter it out using the whitelist during the commit.
      add(propKey, nextProp);
    }
  });

  return updatePayload;
}

function updateProps(element, updateQueue) {
  var qmlElement = element.value;

  updateQueue.forEach(function (_ref7) {
    var _ref8 = _slicedToArray(_ref7, 2),
        propKey = _ref8[0],
        propValue = _ref8[1];

    if (propValue == null || propKey === '__qmlRawContent') {
      // ignore
      return;
    }

    if (propKey === 'children') {
      element.children = propValue;
      return;
      // const { defaultProp } = element;
      // console.log(require('util').inspect(propValue, { depth: 1, colors: true }));
      // setChildren(qmlElement, defaultProp, propValue);
      // return;
    }

    if (propKey.match(isEventRegex)) {
      var match = propKey.match(isEventRegex);

      var _propValue = _slicedToArray(propValue, 2),
          lastHandler = _propValue[0],
          nextHandler = _propValue[1];

      listenTo(qmlElement, match[1], nextHandler, lastHandler);
      return;
    }

    if ((typeof propValue === 'undefined' ? 'undefined' : _typeof(propValue)) === 'object') {
      if (qmlElement[propKey]) {
        entries(propValue).forEach(function (_ref9) {
          var _ref10 = _slicedToArray(_ref9, 2),
              configKey = _ref10[0],
              configValue = _ref10[1];

          qmlElement[propKey][configKey] = configValue;
        });
      }

      return;
    }

    qmlElement[propKey] = propValue;
  });
}