'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _reactReconciler = require('react-reconciler');

var _reactReconciler2 = _interopRequireDefault(_reactReconciler);

var _Registry = require('./Registry');

var _QMLComponent = require('./QMLComponent');

var _objectFactory = require('./object-factory');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var QMLRenderer = (0, _reactReconciler2.default)({
  createInstance: function createInstance(type, props, rootInstance) {
    console.log('createInstance ------------', type);

    if (type === 'attribute') {
      var name = props.name;

      return (0, _objectFactory.makeAttributeNode)(name);
    }

    if (_Registry.Registry[type]) {
      var _Registry$type = _Registry.Registry[type],
          qmlContent = _Registry$type.qmlContent,
          defaultProp = _Registry$type.defaultProp;

      return (0, _objectFactory.makeElementNode)(type, defaultProp, qmlContent, rootInstance);
    }

    throw new Error('unknown type ' + type);
  },
  appendInitialChild: function appendInitialChild(parent, child) {
    console.log('appendInitialChild');
    console.log('  parent: ' + parent.name + '#' + parent.id);
    console.log('  child: ' + child.name + '#' + child.id);
    // creating an attribute node
    if ((0, _objectFactory.isAttribute)(parent)) {
      parent.value = child;
      return;
    }

    // assigning an attribute node
    if ((0, _objectFactory.isAttribute)(child)) {
      var prevValue = parent.value[child.name];

      console.log('  assigning attr', child.name, require('util').inspect(child, { depth: 0, colors: true }));
      (0, _objectFactory.setAttribute)(parent, child.name, child);
    }

    // assign a child element to its parent
    // at this point we're only assign a child node to a parent node
    // notice that node.value is the actual QML Object
    parent.children.push(child);
  },
  finalizeInitialChildren: function finalizeInitialChildren(element, type, props, rootContainerInstance) {
    console.log('finalizeInitialChildren ---', element.name, element.id);
    // attribute node doesn't need finalization
    if ((0, _objectFactory.isAttribute)(element)) {
      return;
    }

    (0, _QMLComponent.setInitialProps)(element.value, props);

    // child-less elements don't need finalization as well
    if (!element.children || element.children.length === 0) {
      return;
    }

    console.log('  children: ', element.children.length);
    element.children.forEach(function (child) {
      console.log('  -', child.name);
    });

    (0, _objectFactory.setAttribute)(element, element.defaultProp, element.children);
  },
  createTextInstance: function createTextInstance(text, rootContainerInstance) {
    console.log('createTextInstance');
    // TODO
    return getOwnerDocument(rootContainerInstance).createTextNode(text);
  },
  getPublicInstance: function getPublicInstance(inst) {
    console.log('getPublicInstance', inst.name);
    if (!(0, _objectFactory.isElement)(inst) && !(0, _objectFactory.isAttribute)(inst)) {
      console.log('invalid instance', require('util').inspect(inst, { depth: 0, colors: true }));
      throw new Error('invalid instance');
    }
    return inst.value;
  },
  prepareForCommit: function prepareForCommit() {
    console.log('prepareForCommit');
    // noop
  },
  prepareUpdate: function prepareUpdate(domElement, type, oldProps, newProps) {
    console.log('prepareUpdate');
    return (0, _QMLComponent.diffProps)(domElement.value, oldProps, newProps);
  },
  resetAfterCommit: function resetAfterCommit() {
    console.log('resetAfterCommit');
    // noop
  },
  resetTextContent: function resetTextContent(domElement) {
    console.log('resetTextContent');
    domElement.textContent = '';
  },
  getRootHostContext: function getRootHostContext(instance) {
    console.log('getRootHostContext', instance);
    if ((typeof instance === 'undefined' ? 'undefined' : _typeof(instance)) !== undefined) {
      return instance;
    }
    return {};
  },
  getChildHostContext: function getChildHostContext(instance) {
    // console.log('getChildHostContext');
    return {};
  },
  shouldSetTextContent: function shouldSetTextContent(type, props) {
    // console.log('shouldSetTextContent ---', type);
    // return (
    //   type === 'textarea' ||
    //   typeof props.data === 'string' ||
    //   typeof props.data === 'number' ||
    //   (typeof props.dangerouslySetInnerHTML === 'object' &&
    //     props.dangerouslySetInnerHTML !== null &&
    //     typeof props.dangerouslySetInnerHTML.__html === 'string')
    // );
    return false;
  },


  now: function now() {},

  useSyncScheduling: true,

  mutation: {
    appendChild: function appendChild(parent, child) {
      console.log('appendChild');
      console.log('  parent', parent.id, parent.name);
      console.log('  child', child.id, child.name);

      var propKey = (0, _objectFactory.isElement)(child) ? parent.defaultProp : child.name;
      (0, _objectFactory.setAttribute)(parent, propKey, child);
    },
    appendChildToContainer: function appendChildToContainer(containerQmlElement, topLevelChild) {
      console.log('appendChildToContainer');
      containerQmlElement.data.push(topLevelChild.value);
    },
    removeChild: function removeChild(parent, child) {
      console.log('removeChild');
      console.log('  parent: ' + parent.type + ' ' + parent.name + '#' + parent.id);
      console.log('  child: ' + child.type + ' ' + child.name + '#' + child.id);

      var propKey = (0, _objectFactory.isElement)(child) ? parent.defaultProp : child.name;

      console.log('  propKey', propKey);

      // assuming propKey is always an array prop
      var list = parent.value[propKey];

      if (typeof list.push === 'function') {
        var index = findIndex(list, function (elm) {
          return elm === child.value;
        });
        var removingElm = list[index];
        parent.value[propKey] = without(list, removingElm);

        // TODO: move this back to pool
        (0, _objectFactory.release)(removingElm);
      } else {
        console.log('  removing attr', propKey, require('util').inspect(child, { depth: 0 }));
        parent.value[propKey] = null;
      }
    },
    removeChildFromContainer: function removeChildFromContainer(containerQmlElement, child) {
      console.log('removeChildFromContainer ---', containerQmlElement, child.value);
      for (var i = containerQmlElement.data.length; i > 0; i--) {
        if (child == containerQmlElement.data[i]) {
          containerQmlElement.data[i].destroy();
          break;
        }
      }
    },
    insertBefore: function insertBefore(parent, child, beforeChild) {
      console.log('insertBefore');
      var propKey = (0, _objectFactory.isElement)(child) ? parent.defaultProp : child.name;

      console.log('  propKey', propKey);

      // assuming propKey is always an array prop
      var list = toArray(parent.value[propKey]);
      var beforeChildIndex = findIndex(list, function (elem) {
        return elem === beforeChild.value;
      });
      list.splice(beforeChild - 1, 0, child.value);

      parent.value[propKey] = list;

      console.log('  length aftermath', list.length);
    },
    commitUpdate: function commitUpdate(instance, preparedUpdateQueue) {
      console.log('commitUpdate');
      (0, _QMLComponent.updateProps)(instance, preparedUpdateQueue);
    },
    commitMount: function commitMount(instance, updatePayload, type, oldProps, newProps) {
      console.log('commitMount');
      // noop
    },
    resetTextContent: function resetTextContent(domElement) {
      console.log('resetTextContent');
      domElement.textContent = '';
    },
    commitTextUpdate: function commitTextUpdate(textInstance, oldText, newText) {
      console.log('commitTextUpdate');
      textInstance.nodeValue = newText;
    }
  }
});

exports.default = QMLRenderer;